<?php

/*
 * Discussion Board Status class
 * @since 1.6.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


class CT_DB_Pro_Status {

	public function __construct() {
		//
	}

	/**
	 * Initialize the class and start calling our hooks and filters
	 * @since 1.6.0
	 */
	public function init() {
		add_action( 'wp_ajax_update_topic_status', array( $this, 'update_topic_status' ) );
		add_filter( 'ctdb_topics_form_before_submit', array( $this, 'topic_form_filter' ) );
		add_action( 'ctdb_new_topic_id', array( $this, 'topic_form_set_status' ), 10, 1 );
		
		// Board taxonomy fields
		add_action( 'board_add_form_fields', array ( $this, 'add_status_meta' ), 10, 2 );
		add_action( 'created_board', array ( $this, 'save_status_meta' ), 10, 2 );
		add_action( 'board_edit_form_fields', array( $this, 'update_status_meta' ), 10, 2 );
		add_action( 'edited_board', array( $this, 'updated_status_meta' ), 10, 2 );
		
		// Filter meta fields
		add_filter( 'ctdb_meta_data_fields_filter', array( $this, 'filter_meta_data_fields' ), 10, 1 );
	}	
	
	/**
	 * Update the topic's status when the select box is updated
	 * @since 1.6.0
	 */
	public function update_topic_status() {
		check_ajax_referer( 'topic_status_nonce', 'security' );
		$return = 'fail';
		if ( ! empty( $_REQUEST['userid'] ) && ! empty( $_REQUEST['postid'] ) ) {
			$userid = absint( $_REQUEST['userid'] );
			$postid = absint( $_REQUEST['postid'] );
			$new_status = stripslashes( strip_tags( $_REQUEST['new_status'] ) );
			
			// We need to update the post meta field
			$current_status = get_post_meta( $postid, 'ctdb_topic_status', true );
			if( $new_status != $current_status ) {
				update_post_meta( $postid, 'ctdb_topic_status', sanitize_text_field( $new_status ) );
			}
			$return = $new_status;
		}
		echo $return;
		die();
	}
	
	// Add statuses to New Topic Form
	public function topic_form_filter( $form ) {
		
		// If poster can't set status, bounce
		$pro_options = get_option( 'ctdb_pro_settings');
		if( empty( $pro_options['poster_set_status'] ) ) {
			return $form;
		}
		// Only include the Status if the scope is Global
		if( ctdb_status_scope() != 'global' ) {
			return $form;
		}
		
		// Display status field if statuses are enabled
		if( ctdb_is_status_enabled() ) {
			
			$statuses = ctdb_statuses_list();
			
			// Add statuses if they exist
			if ( ! empty ( $statuses ) && ! is_wp_error ( $statuses ) ) {
				$status_select = '<div><select name="new_topic_status" id="new_topic_status" class="postform ctdb_new_topic_select">';
				foreach ( $statuses as $status ) {
					$status_select .= '<option value="' . esc_attr( $status ) . '">' . esc_attr( $status ) . '</option>';
				}
				$status_select .= '</select></div>';	
			}

			// Add the select box to the $fields
			if( isset( $status_select ) && $status_select != '' ) {
				$form['fields'][] = $status_select;
			}
			
		}
		
		return $form;
		
	}
	
	/**
	 * Set the status for the topic after it's been posted
	 * @hooked to ctdb_new_topic_id
	 * @since 1.6.0
	*/
	public function topic_form_set_status( $post_id ) {
		
		// Display status field if statuses are enabled
		if( ctdb_is_status_enabled() ) {
			
			// Check if a status has been passed in
			if( isset( $_POST['new_topic_status'] ) ) {
				update_post_meta( $post_id, 'ctdb_topic_status', sanitize_text_field( $_POST['new_topic_status'] ) );
			}
			
		}
		
	}
	
	/**
	 * Add a form field to the Board taxonomy admin page
	 * @since 1.6.0
	 */
	public function add_status_meta( $taxonomy ) { 
		// Only add metafields if status scope is board 
		if( ctdb_status_scope() != 'board' ) {
			return;
		}
		// Get default status list
		$pro_options = get_option( 'ctdb_pro_settings');
		$statuses = '';
		if( isset( $pro_options['default_statuses'] ) ) {
			$statuses = $pro_options['default_statuses'];
		} ?>
		<div class="form-field term-group">
			<label for="ctdb_board_statuses"><?php _e( 'Statuses', 'discussion-board-pro' ); ?></label>
			<input type="text" id="ctdb_board_statuses" name="ctdb_board_statuses" value="<?php echo esc_attr( $statuses ); ?>">
			<p class="description"><?php _e( 'Comma-separated list of statuses. Put your default value first, e.g. <code>Open, Fixed</code>.', 'discussion-board-pro' ); ?></p>
		</div>
	<?php
	}
	
	/**
	 * Save the form field
	 * @since 1.6.0
	 */
	public function save_status_meta( $term_id, $tt_id ) {
		if( isset( $_POST['ctdb_board_statuses'] ) && '' !== $_POST['ctdb_board_statuses'] ){
			add_term_meta( $term_id, 'ctdb_board_statuses', sanitize_text_field( $_POST['ctdb_board_statuses'] ), true );
		}
	}
	
	/**
	 * Edit the form field
	 * @since 1.6.0
	 */
	public function update_status_meta( $term, $taxonomy ) {
		// Only add metafields if status scope is board 
		if( ctdb_status_scope() != 'board' ) {
			return;
		} ?>
		<tr class="form-field term-group-wrap">
			<th scope="row">
				<label for="ctdb_board_statuses"><?php _e( 'Statuses', 'discussion-board-pro' ); ?></label>
			</th>
			<td>
				<input type="text" id="ctdb_board_statuses" name="ctdb_board_statuses" value="">
				<p class="description"><?php _e( 'Comma-separated list of statuses. Put your default value first, e.g. <code>Open, Fixed</code>.', 'discussion-board-pro' ); ?></p>
			</td>
		</tr>
	<?php
	}
	
	/**
	 * Update the form field value
	 * @since 1.5.0
	 */
	public function updated_status_meta( $term_id, $tt_id ) {
		if( isset( $_POST['ctdb_board_statuses'] ) && '' !== $_POST['ctdb_board_statuses'] ){
			update_term_meta( $term_id, 'ctdb_board_statuses', sanitize_text_field( $_POST['ctdb_board_statuses'] ) );
		} else {
			update_term_meta( $term_id, 'ctdb_board_statuses', '' );
		}
	}
	
	/**
	 * Filter meta data fields
	 * We're looking to see if the Status metafield has been selected
	 * If the scope is Board then check that the current Board has statuses enabled
	 * @since 1.6.0
	 */
	public function filter_meta_data_fields( $fields ) {
		// If scope is Global then status is allowed
		if( ctdb_status_scope() == 'global' ) {
			return $fields;
		}
		// If scope is Board, check whether there are statuses for this board
		$board_id = ctdb_topic_board_id();
		// Check if board has statuses
		$statuses = get_term_meta( $board_id, 'ctdb_board_statuses', true );
		if( empty( $statuses ) ) {
			unset( $fields['status'] );
		}
		return $fields;
	}
	
	
}

$CT_DB_Pro_Status = new CT_DB_Pro_Status();
$CT_DB_Pro_Status->init();